import on from '@thinkeloquent/rd2020-dom/src/element/on';
import addClass from '@thinkeloquent/rd2020-dom/src/element/addClass';
import removeClass from '@thinkeloquent/rd2020-dom/src/element/removeClass';
import { findWithDocument } from '@thinkeloquent/rd2020-dom/src/selector/find';

import uid from './uid';

const handleHoverOut = event => {
  let el = event ? event.target : previousElement;
  if (el) removeClass(el, outlineClassName);
};

const outlineClassName = 'outliner5000';
const outlineSelectedClassName = `${outlineClassName}-selected`;
const headStyleTagsId = 'outliner-5000-style-block';

const headStyleTags = `
<style id="${headStyleTagsId}">
    .${outlineClassName} {
        outline: 2px solid red;
    }
    .${outlineSelectedClassName}{
        outline:2px solid blue;
    }
</style>
`;

class ElementHighlight {
  constructor() {
    this.previousHoveredElement = null;
  }

  attachHeadStyles(documentHead) {
    documentHead.insertAdjacentHTML('beforeend', headStyleTags);
  }

  attachedBodyListeners(documentBody, options = {}) {
    let disposeOf;
    const { onClick } = options;
    const hasOnClick = typeof(onClick) === 'function';

    const removePreviousSelection = () => {
      let currentlySelected = findWithDocument(documentBody, `.${outlineSelectedClassName}`);
      if (currentlySelected && currentlySelected.length) {
          removeClass(currentlySelected, outlineSelectedClassName);
      }
      return currentlySelected;
    };

    const disposeOfMouseOver = on(documentBody, 'mouseover', (event) => {
        addClass(event.target, outlineClassName);
        this.previousHoveredElement = event.target;
    });

    const disposeOfMouseOut = on(documentBody, 'mouseout', (event) => {
        const el = event ? event.target : this.previousHoveredElement;
        if (el) removeClass(el, outlineClassName);
    });

    const disposeOfClick = on(documentBody, 'click', (event) => {
      event.preventDefault();
      event.stopPropagation();
      const previousSelectedElement = removePreviousSelection();
      if(hasOnClick) onClick(event.target, previousSelectedElement);
      addClass(event.target, outlineSelectedClassName);
    });

    disposeOf = () => {
      disposeOfMouseOver();
      disposeOfMouseOut();
      disposeOfClick();
    };

    return disposeOf;
  }

  getData() {
    return {
      dom: { ...this.dom },
      draft: { ...this.draft },
      originalText: { ...this.originalText },
    }
  }

}

export default ElementHighlight;